@@ -62,6 +62,272 @@ char extchar[20];
 double saveratio;
 unsigned char texttype;
 
+/*--------------------------------------*/
+/* Element constructor functions	*/
+/*--------------------------------------*/
+
+/*--------------------------------------------------------------*/
+/* Label constructor:  Create a new label element in the object	*/
+/* whose instance is "destinst" and return a pointer to it.	*/
+/*								*/
+/*	"destinst" is the destination instance.  If NULL, the	*/
+/*	   top-level instance (areastruct.topinstance) is used.	*/
+/*	"strptr" is a pointer to a stringpart string, and may	*/
+/*	   be NULL.  If non-NULL, should NOT be free'd by the	*/
+/*	   calling routine.					*/
+/*	"pintype" is NORMAL, LOCAL, GLOBAL, or INFO		*/
+/*	"x" and "y" are the label coordinates.			*/
+/*								*/
+/* Other properties must be set individually by the calling	*/
+/* 	routine.						*/
+/*								*/
+/* Return value is a pointer to the newly created label.	*/
+/*--------------------------------------------------------------*/
+
+labelptr new_label(objinstptr destinst, stringpart *strptr, int pintype,
+	int x, int y)
+{
+   labelptr *newlab;
+   objectptr destobject;
+   objinstptr locdestinst;
+
+   locdestinst = (destinst == NULL) ? areastruct.topinstance : destinst;
+   destobject = locdestinst->thisobject;
+
+   NEW_LABEL(newlab, destobject);
+   destobject->parts++;
+   labeldefaults(*newlab, pintype, x, y);
+
+   if (strptr->type == FONT_NAME) {
+      free ((*newlab)->string);
+      (*newlab)->string = strptr;
+   }
+   else
+      (*newlab)->string->nextpart = strptr;
+
+   calcbboxvalues(locdestinst, (genericptr *)newlab);
+   updatepagebounds(destobject);
+   incr_changes(destobject);
+   return *newlab;
+}
+
+/*--------------------------------------------------------------*/
+/* Variant of the above; creates a label from a (char *) string	*/
+/* instead of a stringpart pointer.  Like the stringpart 	*/
+/* pointer above, "cstr" should NOT be free'd by the calling	*/
+/* routine.							*/
+/*--------------------------------------------------------------*/
+
+labelptr new_simple_label(objinstptr destinst, char *cstr,
+	int pintype, int x, int y)
+{
+   stringpart *strptr;
+
+   strptr = (stringpart *)malloc(sizeof(stringpart));
+   strptr->type = TEXT_STRING;
+   strptr->nextpart = NULL;
+   strptr->data.string = cstr;
+
+   return new_label(destinst, strptr, pintype, x, y);
+}
+
+/*--------------------------------------------------------------*/
+/* Another variant of the above; creates a "temporary" label	*/
+/* from a (char *) string.  As above, "cstr" should NOT be	*/
+/* free'd by the calling routine.  The "temporary" label has no	*/
+/* font information, and cannot be displayed nor saved/loaded	*/
+/* from the PostScript output file.  Used to name networks or	*/
+/* to mark port positions.  Pin type is always LOCAL.  Does not	*/
+/* require updating bounding box info since it cannot be	*/
+/* displayed.  Consequently, only requires passing the object	*/
+/* to get the new label, not its instance.			*/
+/*--------------------------------------------------------------*/
+
+labelptr new_temporary_label(objectptr destobject, char *cstr,
+	int x, int y)
+{
+   labelptr *newlab;
+
+   NEW_LABEL(newlab, destobject);
+   destobject->parts++;
+   labeldefaults(*newlab, LOCAL, x, y);
+
+   (*newlab)->string->type = TEXT_STRING; /* overwrites FONT record */
+   (*newlab)->string->data.string = cstr;
+   return *newlab;
+}
+
+/*--------------------------------------------------------------*/
+/* Polygon constructor:  Create a new polygon element in the	*/
+/* object whose instance is "destinst" and return a pointer to  */
+/* it.								*/
+/*								*/
+/*	"destinst" is the destination instance.  If NULL, the	*/
+/*	   top-level instance (areastruct.topinstance) is used.	*/
+/*	"points" is a list of XPoint pointers, should not be	*/
+/*	   NULL.  It is transferred to the polygon verbatim,	*/
+/*	   and should NOT be free'd by the calling routine.	*/
+/*	"number" is the number of points in the list, or zero	*/
+/*	   if "points" is NULL.					*/
+/*								*/
+/* Other properties must be set individually by the calling	*/
+/* 	routine.						*/
+/*								*/
+/* Return value is a pointer to the newly created polygon.	*/
+/*--------------------------------------------------------------*/
+
+polyptr new_polygon(objinstptr destinst, pointlist *points, int number)
+{
+   polyptr *newpoly;
+   objectptr destobject;
+   objinstptr locdestinst;
+
+   locdestinst = (destinst == NULL) ? areastruct.topinstance : destinst;
+   destobject = locdestinst->thisobject;
+
+   NEW_POLY(newpoly, destobject);
+   destobject->parts++;
+   polydefaults(*newpoly, 0, 0, 0);
+   (*newpoly)->number = number;
+   (*newpoly)->points = *points;
+
+   calcbboxvalues(locdestinst, (genericptr *)newpoly);
+   updatepagebounds(destobject);
+   incr_changes(destobject);
+   return *newpoly;
+}
+
+/*--------------------------------------------------------------*/
+/* Spline constructor:  Create a new spline element in the	*/
+/* object whose instance is "destinst" and return a pointer to  */
+/* it.								*/
+/*								*/
+/*	"destinst" is the destination instance.  If NULL, the	*/
+/*	   top-level instance (areastruct.topinstance) is used.	*/
+/*	"points" is a array of 4 XPoints; should not be	NULL.	*/
+/*								*/
+/* Other properties must be set individually by the calling	*/
+/* 	routine.						*/
+/*								*/
+/* Return value is a pointer to the newly created spline.	*/
+/*--------------------------------------------------------------*/
+
+splineptr new_spline(objinstptr destinst, pointlist points)
+{
+   splineptr *newspline;
+   objectptr destobject;
+   objinstptr locdestinst;
+   int i;
+
+   locdestinst = (destinst == NULL) ? areastruct.topinstance : destinst;
+   destobject = locdestinst->thisobject;
+
+   NEW_SPLINE(newspline, destobject);
+   destobject->parts++;
+   splinedefaults(*newspline, 0, 0);
+
+   for (i = 0; i < 4; i++)
+      (*newspline)->ctrl[i] = points[i];
+
+   calcspline(*newspline);
+   calcbboxvalues(locdestinst, (genericptr *)newspline);
+   updatepagebounds(destobject);
+   incr_changes(destobject);
+   return *newspline;
+}
+
+/*--------------------------------------------------------------*/
+/* Arc constructor:  Create a new arc element in the object	*/
+/* whose instance is "destinst" and return a pointer to it.	*/
+/*								*/
+/*	"destinst" is the destination instance.  If NULL, the	*/
+/*	   top-level instance (areastruct.topinstance) is used.	*/
+/*	"radius" is the radius of the (circular) arc.		*/
+/*	"x" and "y" represents the arc center position.		*/
+/*								*/
+/* Other properties must be set individually by the calling	*/
+/* 	routine.						*/
+/*								*/
+/* Return value is a pointer to the newly created arc.		*/
+/*--------------------------------------------------------------*/
+
+arcptr new_arc(objinstptr destinst, int radius, int x, int y)
+{
+   arcptr *newarc;
+   objectptr destobject;
+   objinstptr locdestinst;
+
+   locdestinst = (destinst == NULL) ? areastruct.topinstance : destinst;
+   destobject = locdestinst->thisobject;
+
+   NEW_ARC(newarc, destobject);
+   destobject->parts++;
+   arcdefaults(*newarc, x, y);
+   (*newarc)->radius = (*newarc)->yaxis = radius;
+
+   calcarc(*newarc);
+   calcbboxvalues(locdestinst, (genericptr *)newarc);
+   updatepagebounds(destobject);
+   incr_changes(destobject);
+   return *newarc;
+}
+
+/*--------------------------------------------------------------*/
+/* Instance constructor:  Create a new object instance element	*/
+/* in the object whose instance is "destinst" and return a	*/
+/* pointer to it.						*/
+/*								*/
+/*	"destinst" is the destination instance.  If NULL, the	*/
+/*	   top-level instance (areastruct.topinstance) is used.	*/
+/*	"srcinst" is the source instance of which this is a	*/
+/*	   copy.						*/
+/*	"x" and "y" represents the instance position.		*/
+/*								*/
+/* Other properties must be set individually by the calling	*/
+/* 	routine.						*/
+/*								*/
+/* Return value is a pointer to the newly created arc.		*/
+/*--------------------------------------------------------------*/
+
+objinstptr new_objinst(objinstptr destinst, objinstptr srcinst, int x, int y)
+{
+   objinstptr *newobjinst;
+   objectptr destobject;
+   objinstptr locdestinst;
+
+   locdestinst = (destinst == NULL) ? areastruct.topinstance : destinst;
+   destobject = locdestinst->thisobject;
+
+   NEW_OBJINST(newobjinst, destobject);
+   destobject->parts++;
+   instcopy(*newobjinst, srcinst);
+   (*newobjinst)->position.x = x;
+   (*newobjinst)->position.y = y;
+
+   calcbboxvalues(locdestinst, (genericptr *)newobjinst);
+   updatepagebounds(destobject);
+   incr_changes(destobject);
+   return *newobjinst;
+}
+
+/*--------------------------------------------------------------*/
+/* Generic element destructor function				*/
+/*--------------------------------------------------------------*/
+
+void remove_element(objinstptr destinst, genericptr genelem)
+{
+   objectptr destobject;
+   objinstptr locdestinst;
+
+   locdestinst = (destinst == NULL) ? areastruct.topinstance : destinst;
+   destobject = locdestinst->thisobject;
+
+   genelem->type &= REMOVE_TAG;
+   delete_tagged(destobject);
+   calcbboxvalues(locdestinst, (genericptr *)NULL);
+   updatepagebounds(destobject);
+}
+
 /*-------------------------------------*/
 /* Sane values for a new path instance */
 /*-------------------------------------*/
@@ -179,6 +445,7 @@ void labeldefaults(labelptr newlabel, u_char dopin, int x, int y)
    if (areastruct.schemon) {
       newlabel->pin = dopin;
       if (dopin == LOCAL) newlabel->color = LOCALPINCOLOR;
+      else if (dopin == GLOBAL) newlabel->color = GLOBALPINCOLOR;
       else if (dopin == INFO) newlabel->color = INFOLABELCOLOR;
    }
 
@@ -936,12 +1203,17 @@ void polydefaults(polyptr newpoly, int number, int x, int y)
    newpoly->color = areastruct.color;
    newpoly->width = areastruct.linewidth;
    newpoly->number = number;
-   newpoly->points = (pointlist) malloc(number * sizeof(XPoint));
    newpoly->passed = NULL;
-   for (pointptr = newpoly->points; pointptr < newpoly->points + number;
-	pointptr++) {
-      pointptr->x = x;
-      pointptr->y = y;
+   if (number == 0)
+      newpoly->points = NULL;
+   else {
+      newpoly->points = (pointlist) malloc(number * sizeof(XPoint));
+
+      for (pointptr = newpoly->points; pointptr < newpoly->points + number;
+		pointptr++) {
+         pointptr->x = x;
+         pointptr->y = y;
+      }
    }
 }
 
