@@ -1215,6 +1215,9 @@ void copycat()
       (*newobj)->viewscale = oldobj->viewscale;
       /* don't attach the same schematic. . . */
       (*newobj)->symschem = NULL;
+      /* don't copy highlights */
+      (*newobj)->highlight.netlist = NULL;
+      (*newobj)->highlight.thisinst = NULL;
 
       /* Copy the parameter structure */
       (*newobj)->params = NULL;
