@@ -120,6 +120,24 @@ void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
    if (mode) drawarea(NULL, NULL, NULL);
 }
 
+/*----------------------------------------------------------------------*/
+/* Find the page object and instance with the indicated name.		*/
+/*----------------------------------------------------------------------*/
+
+objectptr NameToPageObject(char *objname, objinstptr *ret_inst)
+{
+   int i;
+
+   for (i = 0; i < xobjs.pages; i++) {
+      if (xobjs.pagelist[i]->pageinst == NULL) continue;
+      if (!strcmp(objname, xobjs.pagelist[i]->pageinst->thisobject->name)) {
+	 if (ret_inst) *ret_inst = xobjs.pagelist[i]->pageinst;
+	 return xobjs.pagelist[i]->pageinst->thisobject;
+      }
+   }
+   return NULL;
+}
+
 /*----------------------------------------------------------------------*/
 /* Find the object with the indicated name.				*/
 /*----------------------------------------------------------------------*/
@@ -138,17 +156,10 @@ objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
       }
    }
 
-   if (dopages) {
-      for (i = 0; i < xobjs.pages; i++) {
-	 if (xobjs.pagelist[i]->pageinst == NULL) continue;
-	 if (!strcmp(objname, xobjs.pagelist[i]->pageinst->thisobject->name)) {
-	    if (ret_inst) *ret_inst = xobjs.pagelist[i]->pageinst;
-	    return xobjs.pagelist[i]->pageinst->thisobject;
-	 }
-      }
-   }
-   
-   return NULL;
+   if (dopages)
+      return NameToPageObject(objname, ret_inst);
+   else
+      return NULL;
 }
 
 /*------------------------------------------------------------------------*/
@@ -232,13 +243,14 @@ void startconnect(xcWidget button, caddr_t clientdata, caddr_t calldata)
 
 void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
-   short *gsel;
-   genericptr ggen;
+   short *gsel = NULL;
+   selection *rselect = NULL, *nextselect;
+   genericptr ggen = NULL;
    Genericlist *netlist = NULL;
    int depth, lbus, netid, subnetid;
    buslist *sbus;
    pushlistptr seltop, nextptr;
-   objectptr nettop, thisobj;
+   objectptr nettop, thisobj, pschem;
    char *snew;
    stringpart *ppin;
 
@@ -264,12 +276,23 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	    if (!nonnetwork(gpoly)) break;
 	 }
       }
-      if (gsel == areastruct.selectlist + areastruct.selects) {
-	 Wprintf("Selected element is not part of a valid network.");
+   }
+   if ((areastruct.selects == 0) || (gsel == areastruct.selectlist
+		+ areastruct.selects)) {
+      rselect = recurselect(POLYGON | LABEL | OBJINST, MODE_CONNECT, &seltop);
+      /* Should look at the top scorer, when scoring has been implemented */
+      if (rselect && (rselect->selects > 0)) {
+	 for (nextselect = rselect; (nextselect->next != NULL) &&
+		(nextselect->selects > 0); nextselect = nextselect->next);
+         ggen = *(nextselect->thisinst->thisobject->plist + *(nextselect->selectlist));
+	 while (rselect != NULL) {
+	    nextselect = rselect->next;
+	    free(rselect);
+            free(rselect->selectlist);
+	    rselect = nextselect;
+	 }
       }
    }
-   else
-      ggen = recurselect(POLYGON | LABEL, &seltop);
 
    /* Determine the net and the topmost object in which that	*/
    /* net appears.  Then build the transformation matrix down	*/
@@ -296,10 +319,13 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
          while (depth-- > 0) 
 	    UPopCTM();
 
+	 /* get the primary schematic */
+	 pschem = (nettop->schemtype == SECONDARY) ? nettop->symschem : nettop;
+
 	 /* print the net name to the message window */
 
 	 if (netlist->subnets == 0) {
-	    ppin = nettopin(netlist->net.id, nettop, NULL);
+	    ppin = nettopin(netlist->net.id, pschem, NULL);
             snew = textprint(ppin, areastruct.topinstance);
 	    sprintf(_STR2, "Network is \"%s\" in %s", snew, nettop->name);
             free(snew);
@@ -312,7 +338,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	       sbus = netlist->net.list + lbus;
 	       netid = sbus->netid;
 	       subnetid = sbus->subnetid;
-	       ppin = nettopin(netid, nettop, NULL);
+	       ppin = nettopin(netid, pschem, NULL);
                snew = textprintsubnet(ppin, areastruct.topinstance, subnetid);
 	       sprintf(sptr, "%s ", snew);
 	       sptr += strlen(snew) + 1;
@@ -1064,14 +1090,7 @@ void startschemassoc(xcWidget w, pointertype mode, caddr_t calldata)
 
 Boolean schemassoc(objectptr schemobj, objectptr symbolobj)
 {
-   if (eventmode != ASSOC_MODE) {
-      Wprintf("Cannot make (dis)association in this mode");
-#if TCL_WRAPPER
-      Tcl_SetResult(xcinterp, "Cannot make association in this mode.", NULL);
-#endif
-      return False;
-   }
-   else if (schemobj->symschem != NULL || symbolobj->symschem != NULL) {
+   if (schemobj->symschem != NULL || symbolobj->symschem != NULL) {
       Wprintf("Both objects must be disassociated first.");
 #if TCL_WRAPPER
       Tcl_SetResult(xcinterp, "Both objects must be disassociated first.", NULL);
