@@ -232,7 +232,10 @@ int test_insideness(int tx, int ty, XPoint *boxpoints)
 /* Search for selection among path components */
 /*--------------------------------------------*/
 
-Boolean pathselect(genericptr *curgen, short class)
+#define RANGE_NARROW 11.5
+#define RANGE_WIDE 50
+
+Boolean pathselect(genericptr *curgen, short class, float range)
 {
    /*----------------------------------------------------------------------*/
    /* wirelim is the distance, in user-space units, at which an element is */
@@ -246,7 +249,7 @@ Boolean pathselect(genericptr *curgen, short class)
    /*		is an empirical result.					   */
    /*----------------------------------------------------------------------*/
 
-   float	wirelim = 2 + 11.5 / (*areastruct.vscale + 0.05);
+   float	wirelim = 2 + range / (*areastruct.vscale + 0.05);
    long		sqrwirelim = (int)(wirelim * wirelim);
 
    long		newdist;
@@ -335,26 +338,18 @@ Boolean pathselect(genericptr *curgen, short class)
 /* Select one of the elements on-screen */
 /*--------------------------------------*/
 
-short *genselectelement(short class, u_char mode, objectptr selobj, objinstptr selinst)
+selection *genselectelement(short class, u_char mode, objectptr selobj,
+		objinstptr selinst)
 {
-   short	*newselect, *foundtmp, *foundlist;
-
+   selection	*rselect = NULL;
+   short	*newselect;
    genericptr	*curgen;
    XPoint 	newboxpts[4];
-   XEvent 	event;
-   XButtonEvent *buttonevent = (XButtonEvent *)(&event);
-   Boolean	selected, unselect = False;
-   short	found = 0;
+   Boolean	selected;
+   float	range = RANGE_NARROW;
 
-   XDefineCursor(dpy, areastruct.areawin, QUESTION);
-
-   /* Definition for unselecting an element */
-
-   if (class < 0) {
-      unselect = True;
-      class = -class;
-   }
-   class &= areastruct.filter;		/* apply the selection filter */
+   if (mode == MODE_RECURSE_WIDE)
+      range = RANGE_WIDE;
 
    /* Loop through all elements found underneath the cursor */
 
@@ -415,7 +410,7 @@ short *genselectelement(short class, u_char mode, objectptr selobj, objinstptr s
 
  	 for (pathp = TOPATH(curgen)->plist; pathp < TOPATH(curgen)->plist +
 		TOPATH(curgen)->parts; pathp++)
-	    if (pathselect(pathp, SPLINE|ARC|POLYGON)) {
+	    if (pathselect(pathp, SPLINE|ARC|POLYGON, range)) {
 	       selected = True;
 	       areastruct.editsubpart = (short)(pathp - TOPATH(curgen)->plist);
 	       break;
@@ -424,149 +419,28 @@ short *genselectelement(short class, u_char mode, objectptr selobj, objinstptr s
 
       /* Check among polygons, arcs, and curves */
 
-      else selected = pathselect(curgen, class);
-
-      /* check if this object has already been selected */
-
-      for (newselect = areastruct.selectlist; newselect <
-             areastruct.selectlist + areastruct.selects; newselect++) 
-         if (*newselect == (short)(curgen - selobj->plist)) {
-	    if (!unselect) selected = False;
-	    break;
-	 }
-
-      if (unselect && (newselect == areastruct.selectlist + areastruct.selects))
-	 selected = False;
+      else selected = pathselect(curgen, class, range);
 
       /* Add this object to the list of things found under the cursor */
 
-      if (selected && (!unselect)) {
-
-         if (found == 0)
-            foundlist = (short *) malloc(sizeof(short));
-         else
-            foundlist = (short *) realloc(foundlist, (found + 1) * sizeof(short));
-
-         newselect = foundlist + found;
-         found++;
-     
-         if (selected) *newselect = (short)(curgen - selobj->plist);
+      if (selected) {
+         if (rselect == NULL) {
+	    rselect = (selection *)malloc(sizeof(selection));
+	    rselect->selectlist = (short *)malloc(sizeof(short));
+	    rselect->selects = 0;
+	    rselect->thisinst = selinst;
+	    rselect->next = NULL;
+	 }
+         else {
+            rselect->selectlist = (short *)realloc(rselect->selectlist,
+			(rselect->selects + 1) * sizeof(short));
+	 }
+	 *(rselect->selectlist + rselect->selects) = (short)(curgen -
+		selobj->plist);
+	 rselect->selects++;
       }
-
-      /* mechanism for unselecting elements under the cursor */
-
-      else if (selected) {
-         short *desel;
-
-         XSetFunction(dpy, areastruct.gc, GXcopy);
-         XTopSetForeground(GSELTOCOLOR(selobj, newselect));
-         geneasydraw(*newselect, DEFAULTCOLOR, selobj, selinst);
-         areastruct.selects--;
-         for (desel = newselect; desel < areastruct.selectlist +
-	        areastruct.selects; desel++)
-	    *desel = *(desel + 1);
-         if (areastruct.selects == 0) free (areastruct.selectlist);
-      }
-      else newselect = NULL;
-
-   } /* for-loop over all parts */
-
-   /* Various options depending on how many things were found */
-
-   switch (found) {
-      case 0: 
-	 newselect = NULL;
-	 break;
-
-      case 1: 
-	 newselect = allocselect();
-	 *newselect = *foundlist;
-	 free(foundlist);
-	 gendrawselected(newselect, selobj, selinst);
-	 break;
-
-      default: {
-	 newselect = NULL;
-	 XSetFunction(dpy, areastruct.gc, GXcopy);
-	 for (foundtmp = foundlist; foundtmp < foundlist + found; foundtmp++) {
-            XSetForeground(dpy, areastruct.gc, QUERYCOLOR);
-            geneasydraw(*foundtmp, DOFORALL, selobj, selinst);
-	    sprintf(_STR, "Click to accept/reject: %d of %d",
-		(int)(foundtmp - foundlist + 1), found);
-	    Wprintf(_STR);
-
-	    for(;;) {  	/* This flushes the X event queue */
-	       XNextEvent(dpy, &event); 
-	       if (event.type != ButtonPress && event.type != ButtonRelease
-		    && event.type != MotionNotify && event.type != KeyPress)
-	          xcDispatchEvent(&event);
-	       else if (event.type == ButtonPress) break;
-	    }
-	    if (buttonevent->button == Button3) {
-               XTopSetForeground(GSELTOCOLOR(selobj, foundtmp));
-               geneasydraw(*foundtmp, DEFAULTCOLOR, selobj, selinst);
-	    }
-	    else {
-	       newselect = allocselect();
-               *newselect = *foundtmp;
-               XSetForeground(dpy, areastruct.gc, SELECTCOLOR);
-               geneasydraw(*newselect, DOFORALL, selobj, selinst);
-	    }
-         }
-      }  break;
    }
-
-   if (found > 1) {
-      short *nselect;
-
-      XDefineCursor(dpy, areastruct.areawin, CIRCLE);
-      if (eventmode == PRESS_MODE)
-	 Wprintf("Click and hold button 1 to continue move.");
-      else
-	 Wprintf("Selection complete");
-
-      for(;;) {
-	 XNextEvent(dpy, &event); 
-	 if (event.type != ButtonPress && event.type != ButtonRelease &&
-		event.type != MotionNotify && event.type != KeyPress)
-	    xcDispatchEvent(&event);
-	 else if ((event.type == ButtonPress) && (eventmode == PRESS_MODE))
-	    break;
-	 else if ((event.type == ButtonRelease) && (eventmode != PRESS_MODE))
-	    break;
-      }
-      free(foundlist);
-
-      /* make it work right for smooth xor'ing */
-
-      XSetFunction(dpy, areastruct.gc, GXcopy);
-      XSetForeground(dpy, areastruct.gc, BACKGROUND);
-      for (nselect = areastruct.selectlist; nselect < areastruct.selectlist
-	   + areastruct.selects; nselect++)
-         geneasydraw(*nselect, DOFORALL, selobj, selinst);
-      XSetFunction(dpy, areastruct.gc, GXxor);
-      XSetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
-      for (nselect = areastruct.selectlist; nselect < areastruct.selectlist
-	   + areastruct.selects; nselect++)
-         geneasydraw(*nselect, DOFORALL, selobj, selinst);
-   }
-
-   setoptionmenu();
-
-   if (eventmode == DELETE_MODE)
-      XDefineCursor(dpy, areastruct.areawin, SCISSORS);
-   else
-      XDefineCursor(dpy, areastruct.areawin, CROSS);
-
-   /* snap the coordinates */
-
-   u2u_snap(&areastruct.save);
-
-   /* Register the selection as an undo event */
-   register_for_undo(XCF_Select, mode, areastruct.topinstance,
-		areastruct.selectlist, areastruct.selects);
-
-   return newselect;
+   return rselect;
 }
 
 /*----------------------------------------------------------------*/
@@ -886,8 +760,9 @@ void unselect_all()
    clearselects();
 }
 
+
 /*----------------------------------------------------------------------*/
-/* Select the nearest polygon, searching the hierarchy if necessary.	*/
+/* Select the nearest element, searching the hierarchy if necessary.	*/
 /* Return an pushlist pointer to a linked list containing the 		*/
 /* hierarchy of objects, with the topmost pushlist also containing a	*/
 /* pointer to the polygon found.					*/
@@ -895,13 +770,19 @@ void unselect_all()
 /* the calling routine.							*/
 /*----------------------------------------------------------------------*/
 
-genericptr recurselect(short class, pushlistptr *seltop)
+selection *recurselect(short class, u_char mode, pushlistptr *seltop)
 {
-   genericptr rgen = NULL;
-   short *selectobj;
+   selection *rselect, *rcheck, *lastselect, *nextselect;
+   genericptr rgen;
+   short i, *selectobj, numselect, origselect;
+   short rclass;
    objectptr selobj;
    objinstptr selinst;
    XPoint savesave, tmppt;
+   pushlistptr selnew;
+   short j, unselects;
+   u_char locmode = (mode == MODE_CONNECT) ? UNDO_DONE : mode;
+   u_char recmode = (mode != MODE_CONNECT) ? MODE_RECURSE_WIDE : MODE_RECURSE_NARROW;
 
    if (*seltop == NULL) {
       Fprintf(stderr, "Error: recurselect called with NULL pushlist pointer\n");
@@ -911,49 +792,84 @@ genericptr recurselect(short class, pushlistptr *seltop)
    selinst = (*seltop)->thisinst;
    selobj = selinst->thisobject;
 
-   if ((selectobj = genselectelement(class, UNDO_DONE, selobj, selinst)) != NULL) {
-      rgen = *(selobj->plist + (*selectobj));
-      /* Fprintf(stdout, "%s found in object %s\n", (IS_LABEL(rgen)) ?
-			"label" : "polygon", selobj->name); */
-      draw_normal_selected(selobj, selinst);
-      clearselects();
-   }
-   else if ((selectobj = genselectelement(OBJINST, UNDO_DONE, selobj, selinst)) != NULL) {
-      pushlistptr selnew = (pushlistptr)malloc(sizeof(pushlist));
+   class &= areastruct.filter;		/* apply the selection filter */
 
-      selinst = TOOBJINST(selobj->plist + (*selectobj));	
-      draw_normal_selected(selobj, selinst);
-      clearselects();
+   unselects = 0;
+   rselect = genselectelement(class, locmode, selobj, selinst);
+   if (rselect == NULL) return NULL;
 
-      /* Link hierarchy information to the pushlist linked list */
-      selnew->thisinst = selinst;
-      selnew->next = NULL;
-      (*seltop)->next = selnew;
+   for (i = 0; i < rselect->selects; i++) {
+      rgen = *(selobj->plist + (*(rselect->selectlist + i)));
+      if (rgen->type == OBJINST) {
+         selinst = TOOBJINST(selobj->plist + (*(rselect->selectlist + i)));	
+
+         /* Link hierarchy information to the pushlist linked list */
+         selnew = (pushlistptr)malloc(sizeof(pushlist));
+         selnew->thisinst = selinst;
+         selnew->next = NULL;
+         (*seltop)->next = selnew;
 	 
-      /* Translate areastruct.save into object's own coordinate system */
-      savesave.x = areastruct.save.x;
-      savesave.y = areastruct.save.y;
-      InvTransformPoints(&areastruct.save, &tmppt, 1, selinst->position,
-		selinst->scale, selinst->rotation);
-      areastruct.save.x = tmppt.x;
-      areastruct.save.y = tmppt.y;
-      /* Fprintf(stdout, "objinst %s found in object %s; searching recursively\n",
-		selinst->thisobject->name, selobj->name); */
-      /* Fprintf(stdout, "cursor position originally (%d, %d); "
-		"in new object is (%d, %d)\n",
-		savesave.x, savesave.y,
-		areastruct.save.x, areastruct.save.y); */
-      UPushCTM();
-      UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
-      rgen = recurselect(class, &selnew);
-      UPopCTM();
-      areastruct.save.x = savesave.x;
-      areastruct.save.y = savesave.y;
-   }
-   /* else
-      Fprintf(stdout, "nothing found in object %s\n", selobj->name); */
+         /* Translate areastruct.save into object's own coordinate system */
+         savesave.x = areastruct.save.x;
+         savesave.y = areastruct.save.y;
+         InvTransformPoints(&areastruct.save, &tmppt, 1, selinst->position,
+			selinst->scale, selinst->rotation);
+         areastruct.save.x = tmppt.x;
+         areastruct.save.y = tmppt.y;
+         /* Fprintf(stdout, "objinst %s found in object %s; searching recursively\n",
+			selinst->thisobject->name, selobj->name); */
+         /* Fprintf(stdout, "cursor position originally (%d, %d); "
+			"in new object is (%d, %d)\n",
+			savesave.x, savesave.y,
+			areastruct.save.x, areastruct.save.y); */
+         UPushCTM();
+         UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
+	 rclass = class;
+	 if ((class & (~OBJINST)) == 0) class = ALL_TYPES;
+         rcheck = recurselect(class, recmode, &selnew);
+         UPopCTM();
+         areastruct.save.x = savesave.x;
+         areastruct.save.y = savesave.y;
 
-   return rgen;
+	 /* If rgen is NULL, remove selected object from the list, and	*/
+	 /* remove the last entry from the pushlist stack.		*/
+
+	 if (rcheck == NULL) {
+	    *(rselect->selectlist + i) = -1;
+	    unselects++;
+	    (*seltop)->next = NULL;
+	    if (selnew->next != NULL)
+	       Fprintf(stderr, "Error: pushstack was freed, but was not empty!\n");
+	    free(selnew);
+	 }
+	 else {
+	    for (lastselect = rselect; lastselect->next != NULL; lastselect =
+			lastselect->next);
+	    lastselect->next = rcheck;
+	 }
+      }
+   }
+
+   /* Modify the selection list */
+
+   for (i = 0, j = 0; i < rselect->selects; i++) {
+      if (*(rselect->selectlist + i) >= 0) {
+	 if (i != j)
+	    *(rselect->selectlist + j) = *(rselect->selectlist + i);
+	 j++;
+      }
+   }
+   rselect->selects -= unselects;
+   if (rselect->selects == 0) {
+      while (rselect != NULL) {
+         nextselect = rselect->next;
+         free(rselect);
+         free(rselect->selectlist);
+         rselect = nextselect;
+      }
+      rselect = NULL;
+   }
+   return rselect;
 }
 
 /*----------------------------------*/
@@ -1007,4 +923,104 @@ void trackselarea()
    areastruct.save.y = newpos.y;
 }
 
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Recursive selection mechanism					*/
+/*----------------------------------------------------------------------*/
+
+short *recurse_select_element(short class, u_char mode) {
+   pushlistptr seltop, nextptr;
+   selection *rselect, *nextselect;
+   short *newselect, *desel;
+   int i, j, k, ilast;
+   static int pick = 0;
+   Boolean unselect = False;
+
+   seltop = (pushlistptr)malloc(sizeof(pushlist));
+   seltop->thisinst = areastruct.topinstance;
+   seltop->next = NULL;
+
+   /* Definition for unselecting an element */
+
+   if (class < 0) {
+      unselect = True;
+      class = -class;
+   }
+   rselect = recurselect(class, mode, &seltop);
+
+   /* Mechanism for unselecting elements under the cursor	*/
+   /* (Unselect all picked objects)				*/
+
+   if (rselect && unselect) {
+      for (i = 0; i < rselect->selects; i++) {
+	 newselect = rselect->selectlist + i;
+         for (j = 0; j < areastruct.selects; j++) {
+            if (*(areastruct.selectlist + j) == *newselect) {
+               XSetFunction(dpy, areastruct.gc, GXcopy);
+               XTopSetForeground(GSELTOCOLOR(topobject, newselect));
+               geneasydraw(*newselect, DEFAULTCOLOR, topobject, areastruct.topinstance);
+               areastruct.selects--;
+               for (k = j; k < areastruct.selects; k++)
+	          *(areastruct.selectlist + k) = *(areastruct.selectlist + k + 1);
+	       break;
+	    }
+	 }
+      }
+      if (areastruct.selects == 0) {
+	 free (areastruct.selectlist);
+	 areastruct.selectlist = NULL;
+      }
+   }
+   else if (rselect) {
+
+      /* Mechanism for selecting objects:			*/
+      /* Count all elements from rselect that are part of	*/
+      /* the current selection.  Pick the "pick"th item (modulo	*/
+      /* total number of items).				*/
+
+      ilast = -1;
+      k = 0;
+      for (i = 0; i < rselect->selects; i++) {
+	 for (j = 0; j < areastruct.selects; j++) {
+	    if (*(areastruct.selectlist + j) == *(rselect->selectlist + i))
+	       break;
+	 }
+	 if (j == areastruct.selects) {
+	    ilast = i;
+	    if (++k == pick)
+	       break;
+	 }
+      }
+
+      if (ilast >= 0) {
+         newselect = allocselect();
+         *newselect = *(rselect->selectlist + ilast);
+         gendrawselected(newselect, topobject, areastruct.topinstance);
+         setoptionmenu();
+         u2u_snap(&areastruct.save);
+
+         /* Register the selection as an undo event */
+         register_for_undo(XCF_Select, mode, areastruct.topinstance,
+		areastruct.selectlist, areastruct.selects);
+
+         /* Make sure this object is not the first to be picked next time */
+         if (++pick >= rselect->selects) pick = 0;
+      }
+   }
+
+   /* Cleanup */
+
+   while (seltop != NULL) {
+      nextptr = seltop->next;
+      free(seltop);
+      seltop = nextptr;
+   }
+   while (rselect != NULL) {
+      nextselect = rselect->next;
+      free(rselect);
+      free(rselect->selectlist);
+      rselect = nextselect;
+   }
+
+   return areastruct.selectlist;
+}
+
