@@ -47,7 +47,7 @@ typedef struct {
    short number;
    genericptr *element;
    short *idx;
-} selection;
+} uselection;
 
 /*----------------------------------------------------------------------*/
 /* Externally declared variables					*/
@@ -55,22 +55,22 @@ typedef struct {
 
 extern Globaldata xobjs;
 extern Clientdata areastruct;
-short   eventmode;
+extern short eventmode;
 
 /*----------------------------------------------------------------------*/
 /* remember_selection ---						*/
 /*									*/
-/*   Copy a selection list into a "selection" record.  The selection	*/
+/*   Copy a selection list into a "uselection" record.  The uselection	*/
 /*   record maintains the order of the elements as well as pointers to	*/
 /*   each element, so the original element ordering can be recovered.	*/
 /*----------------------------------------------------------------------*/
 
-selection *remember_selection(objinstptr topinst, short *slist, int number)
+uselection *remember_selection(objinstptr topinst, short *slist, int number)
 {
    int i, idx;
-   selection *newlist;
+   uselection *newlist;
 
-   newlist = (selection *)malloc(sizeof(selection));
+   newlist = (uselection *)malloc(sizeof(uselection));
    if (number > 0) {
       newlist->element = (genericptr *)malloc(number * sizeof(genericptr));
       newlist->idx = (short *)malloc(number * sizeof(short));
@@ -89,11 +89,11 @@ selection *remember_selection(objinstptr topinst, short *slist, int number)
 }
 
 /*----------------------------------------------------------------------*/
-/* Create a selection list in areastruct from the saved selection	*/
+/* Create a selection list in areastruct from the saved uselection	*/
 /* record.								*/
 /*----------------------------------------------------------------------*/
 
-short *regen_selection(objinstptr thisinst, selection *srec)
+short *regen_selection(objinstptr thisinst, uselection *srec)
 {
    int i, j, k;
    genericptr egen;
@@ -147,7 +147,7 @@ short *regen_selection(objinstptr thisinst, selection *srec)
 /*   Free memory allocated to an undo record selection list.		*/
 /*----------------------------------------------------------------------*/
 
-void free_selection(selection *selrec)
+void free_selection(uselection *selrec)
 {
    if (selrec->number > 0) {
       free(selrec->element);
@@ -192,7 +192,7 @@ void add_to_selectlist(genericptr thisinst)
 int select_previous(Undoptr thisrecord)
 {
    Undoptr chkrecord;
-   selection *srec;
+   uselection *srec;
 
    unselect_all();
    for (chkrecord = thisrecord->next; chkrecord != NULL; chkrecord = chkrecord->next) {
@@ -208,7 +208,7 @@ int select_previous(Undoptr thisrecord)
 	    return 0;
 	 case XCF_Copy:
 	 case XCF_Select:
-	    srec = (selection *)chkrecord->undodata;
+	    srec = (uselection *)chkrecord->undodata;
 	    areastruct.selectlist = regen_selection(thisrecord->thisinst, srec);
 	    areastruct.selects = (areastruct.selectlist) ? srec->number : 0;
 	    return 0;
@@ -254,7 +254,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    objectptr delobj;
    objinstptr newinst;
    Undoptr newrecord;
-   selection *srec;
+   uselection *srec;
    genericptr egen;
    XPoint *fpoint;
 
@@ -403,7 +403,7 @@ short undo_one_action()
    Undoptr thisrecord, chkrecord;
    objectptr thisobj;
    objinstptr thisinst;
-   selection *srec;
+   uselection *srec;
    short *slist;
    XPoint *delta;
    int i, snum, npage, opage;
@@ -505,7 +505,7 @@ short undo_one_action()
 	 break;
 
       case XCF_Library_Pop:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
 	 thisobj = delete_element(thisrecord->thisinst, slist,
 			srec->number, DRAW);
@@ -514,7 +514,7 @@ short undo_one_action()
 	 break;
 
       case XCF_Copy:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
 	 thisobj = delete_element(thisrecord->thisinst, slist,
 			srec->number, DRAW);
@@ -586,7 +586,7 @@ short redo_one_action()
    genericptr egen;
    short *slist;
    XPoint *delta;
-   selection *srec;
+   uselection *srec;
    int tpage, snum;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
@@ -605,7 +605,7 @@ short redo_one_action()
 
    switch(thisrecord->type) {
       case XCF_Delete:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
 	 thisobj = delete_element(thisrecord->thisinst, slist,
 		srec->number, DRAW);
@@ -618,7 +618,7 @@ short redo_one_action()
 
       /* Unfinished! */
       case XCF_Select_Save:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
 	 break;
 
@@ -636,7 +636,7 @@ short redo_one_action()
 
       case XCF_Select:
 	 unselect_all();
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 areastruct.selectlist = regen_selection(thisrecord->thisinst, srec);
 	 areastruct.selects = (areastruct.selectlist) ? srec->number : 0;
 	 draw_all_selected();
@@ -779,7 +779,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 {
    u_int type;
    objectptr uobj;
-   selection *srec;
+   uselection *srec;
 
    type = thisrecord->type;
    switch (type) {
@@ -789,7 +789,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 	    reset(uobj, DESTROY);
 	 }
 	 else { /* MODE_REDO */
-	    srec = (selection *)thisrecord->undodata;
+	    srec = (uselection *)thisrecord->undodata;
 	    free_selection(srec);
 	 }
 	 break;
@@ -805,7 +805,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
       case XCF_Copy:
       case XCF_Library_Pop:
 	 if (mode == MODE_UNDO) {
-	    srec = (selection *)thisrecord->undodata;
+	    srec = (uselection *)thisrecord->undodata;
 	    free_selection(srec);
 	 }
 	 else { /* MODE_REDO */
@@ -819,7 +819,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 	 break;
 
       case XCF_Select:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 free_selection(srec);
 	 break;
 
